==> pypfilt-0.4.3-py2.py3-none-any.whl: pypfilt/model.py <==
"""Base class for simulation models."""

from __future__ import absolute_import, division, print_function
from __future__ import unicode_literals

import numpy as np


class Base(object):
    """
    The base class for simulation models, which defines the minimal set of
    methods that are required.
    """

    @staticmethod
    def init(params, vec):
        """
        Initialise a matrix of state vectors.

        :param params: Simulation parameters.
        :param vec: An uninitialised :math:`P \\times S` matrix of state
            vectors, for :math:`P` particles and state vectors of length
            :math:`S` (as defined by :py:func:`~state_size`).
            To set, e.g., the first element of each state vector to :math:`1`,
            you can use an ellipsis slice: :code:`vec[..., 0] = 1`.
        :raises NotImplementedError: Derived classes *must* implement this
            method.
        """
        raise NotImplementedError()

    @staticmethod
    def state_size():
        """
        Return the size of the state vector.

        :raises NotImplementedError: Derived classes *must* implement this
            method.
        """
        raise NotImplementedError()

    @staticmethod
    def priors(params):
        """
        Return a dictionary of model parameter priors. Each key must
        identify a parameter by name. Each value must be a function that
        returns samples from the associated prior distribution, and should
        have the following form:

        .. code-block:: python

           lambda r, size=None: r.uniform(1.0, 2.0, size=size)

        Here, the argument ``r`` is a PRNG instance and ``size`` specifies the
        output shape (by default, a single value).

        :param params: Simulation parameters.
        :raises NotImplementedError: Derived classes *must* implement this
            method.
        """
        raise NotImplementedError()

    @classmethod
    def update(cls, params, step_date, dt, is_fs, prev, curr):
        """
        Perform a single time-step.

        :param params: Simulation parameters.
        :param step_date: The date and time of the current time-step.
        :param dt: The time-step size (days).
        :param is_fs: Indicates whether this is a forecasting simulation.
        :param prev: The state before the time-step.
        :param curr: The state after the time-step (destructively updated).
        :raises NotImplementedError: Derived classes *must* implement this
            method.
        """
        raise NotImplementedError()

    @classmethod
    def state_info(cls):
        """
        Describe each state variable as a ``(name, index)`` tuple, where
        ``name`` is a descriptive name for the variable and ``index`` is the
        index of that variable in the state vector.

        :raises NotImplementedError: Derived classes *must* implement this
            method.
        """
        raise NotImplementedError()

    @classmethod
    def param_info(cls):
        """
        Describe each model parameter as a ``(name, index)`` tuple, where
        ``name`` is a descriptive name for the parameter and ``index`` is the
        index of that parameter in the state vector.

        :raises NotImplementedError: Derived classes *must* implement this
            method.
        """
        raise NotImplementedError()

    @classmethod
    def param_bounds(cls):
        """
        Return two arrays that contain the (default) lower and upper bounds,
        respectively, for each model parameter.

        :raises NotImplementedError: Derived classes *must* implement this
            method.
        """
        raise NotImplementedError()

    @classmethod
    def stat_info(cls):
        """
        Describe each statistic that can be calculated by this model as a
        ``(name, stat_fn)`` tuple, where ``name`` is a string that identifies
        the statistic and ``stat_fn`` is a function that calculates the value
        of the statistic.

        :raises NotImplementedError: Derived classes *must* implement this
            method.
        """
        raise NotImplementedError()

    @staticmethod
    def is_valid(hist):
        """
        Identify particles whose state and parameters can be inspected. By
        default, this function returns ``True`` for all particles. Override
        this function to ensure that inchoate particles are correctly
        ignored.
        """
        return np.ones((hist.shape[-2],), dtype=bool)

==> pypfilt-0.5.5-py2.py3-none-any.whl: pypfilt/model.py <==
"""Base class for simulation models."""

from __future__ import absolute_import, division, print_function
from __future__ import unicode_literals

import abc
import numpy as np


# Create a new metaclass for abstract base classes (ABCs), so that subclasses
# can only be instantiated if they implement all of the abstract methods.
# The metaclass syntax differs between Python 2 and 3, the following line is
# the simplest way to sidestep this issue without relying on ``six``.
ABC = abc.ABCMeta(str('ABC'), (object,), {})


class Model(ABC):
    """
    The base class for simulation models, which defines the minimal set of
    methods that are required.
    """

    @abc.abstractmethod
    def init(self, params, vec):
        """
        Initialise a matrix of state vectors.

        :param params: Simulation parameters.
        :param vec: An uninitialised :math:`P \\times S` matrix of state
            vectors, for :math:`P` particles and state vectors of length
            :math:`S` (as defined by :py:func:`~state_size`).
            To set, e.g., the first element of each state vector to :math:`1`,
            you can use an ellipsis slice: :code:`vec[..., 0] = 1`.
        """
        pass

    @abc.abstractmethod
    def state_size(self):
        """
        Return the size of the state vector.
        """
        pass

    @abc.abstractmethod
    def priors(self, params):
        """
        Return a dictionary of model parameter priors. Each key must
        identify a parameter by name. Each value must be a function that
        returns samples from the associated prior distribution, and should
        have the following form:

        .. code-block:: python

           lambda r, size=None: r.uniform(1.0, 2.0, size=size)

        Here, the argument ``r`` is a PRNG instance and ``size`` specifies the
        output shape (by default, a single value).

        :param params: Simulation parameters.
        """
        pass

    @abc.abstractmethod
    def update(self, params, step_date, dt, is_fs, prev, curr):
        """
        Perform a single time-step.

        :param params: Simulation parameters.
        :param step_date: The date and time of the current time-step.
        :param dt: The time-step size (days).
        :param is_fs: Indicates whether this is a forecasting simulation.
        :param prev: The state before the time-step.
        :param curr: The state after the time-step (destructively updated).
        """
        pass

    @abc.abstractmethod
    def describe(self):
        """
        Describe each component of the state vector with a tuple of the form
        ``(name, smooth, min, max)``, where ``name`` is a descriptive name for
        the variable/parameter, ``smooth`` is a boolean that indicates whether
        the parameter admits continuous sampling (e.g., post-regularisation),
        and ``min`` and ``max`` define the (inclusive) range of valid values.
        These tuples **must** be in the same order as the state vector itself.
        """

    def stat_info(self):
        """
        Describe each statistic that can be calculated by this model as a
        ``(name, stat_fn)`` tuple, where ``name`` is a string that identifies
        the statistic and ``stat_fn`` is a function that calculates the value
        of the statistic.
        """
        return []

    def is_valid(self, hist):
        """
        Identify particles whose state and parameters can be inspected. By
        default, this function returns ``True`` for all particles. Override
        this function to ensure that inchoate particles are correctly
        ignored.
        """
        return np.ones((hist.shape[-2],), dtype=bool)
